
var delay = require('../timers/delay'),
    rand = require('../math/rand'),
    toNumber = require('../util/toNumber');

/**
 * Timer function where the tick interval jitters between `min...max` milliseconds. 
 *
 * The `callback` function is passed `interval` (time since the previous tick), `i` (number of ticks), and a `stop()` function.
 * @static
 * @memberof plonk
 * @name dust
 * @param {number} min
 * @param {number} max
 * @param {function} [callback=noop]
 * @returns {promise} 
 * @example
 * plonk.dust(30, 100, function (interval, i, stop) {
 *   if (i === 10) stop();
 * })
 * .progress(function (interval) {
 *   console.log(interval);
 *   // => 74.155273
 *   //    53.998158000000004
 *   //    99.259871
 *   //    53.27543200000002
 *   //    77.56419299999999
 *   //    ...
 * })
 * .then(function (elapsed) {
 *   console.log(elapsed);
 *   // => 663.0071679999999
 * });
 */
module.exports = function (min, max, callback) {
  min = toNumber(min, 10);
  max = toNumber(max, 100);
  if (arguments.length <= 2) {
    max = min;
    min = 0;
  }
  return delay(rand(min, max), function (interval, i, stop) {
    callback && callback(interval, i, stop);
    return rand(min, max);
  });
};
